@@ -104,7 +104,7 @@ struct mdp_superblock_1 {
 #define	MD_FEATURE_ALL			(1|2|4)
 
 #ifndef offsetof
-#define offsetof(t,f) ((int)&(((t*)0)->f))
+#define offsetof(t,f) ((size_t)&(((t*)0)->f))
 #endif
 static unsigned int calc_sb_1_csum(struct mdp_superblock_1 * sb)
 {
@@ -124,8 +124,10 @@ static unsigned int calc_sb_1_csum(struct mdp_superblock_1 * sb)
 	disk_csum = sb->sb_csum;
 	sb->sb_csum = 0;
 	newcsum = 0;
-	for (i=0; size>=4; size -= 4 )
-		newcsum += __le32_to_cpu(*isuper++);
+	for (i=0; size>=4; size -= 4 ) {
+		newcsum += __le32_to_cpu(*isuper);
+		isuper++;
+	}
 
 	if (size == 2)
 		newcsum += __le16_to_cpu(*(unsigned short*) isuper);
@@ -275,6 +277,18 @@ static void examine_super1(void *sbv, char *homehost)
 	default: break;
 	}
 	printf("\n");
+	printf("    Array Slot : %d (", __le32_to_cpu(sb->dev_number));
+	for (i= __le32_to_cpu(sb->max_dev); i> 0 ; i--)
+		if (__le16_to_cpu(sb->dev_roles[i-1]) != 0xffff)
+			break;
+	for (d=0; d < i; d++) {
+		int role = __le16_to_cpu(sb->dev_roles[d]);
+		if (d) printf(", ");
+		if (role == 0xffff) printf("empty");
+		else if(role == 0xfffe) printf("failed");
+		else printf("%d", role);
+	}
+	printf(")\n");
 	printf("   Array State : ");
 	for (d=0; d<__le32_to_cpu(sb->raid_disks); d++) {
 		int cnt = 0;
@@ -309,6 +323,7 @@ static void brief_examine_super1(void *sbv)
 	struct mdp_superblock_1 *sb = sbv;
 	int i;
 	char *nm;
+	char *c=map_num(pers, __le32_to_cpu(sb->level));
 
 	nm = strchr(sb->set_name, ':');
 	if (nm)
@@ -318,8 +333,6 @@ static void brief_examine_super1(void *sbv)
 	else
 		nm = "??";
 
-	char *c=map_num(pers, __le32_to_cpu(sb->level));
-
 	printf("ARRAY /dev/md/%s level=%s metadata=1 num-devices=%d UUID=",
 	       nm,
 	       c?c:"-unknown-", sb->raid_disks);
@@ -715,7 +728,8 @@ static int store_super1(struct supertype *st, int fd, void *sbv)
 			(((char*)sb)+1024);
 		if (__le32_to_cpu(bm->magic) == BITMAP_MAGIC) {
 			locate_bitmap1(st, fd, sbv);
-			write(fd, bm, sizeof(*bm));
+			if (write(fd, bm, sizeof(*bm)) != sizeof(*bm))
+			    return 5;
 		}
 	}
 	fsync(fd);
@@ -765,7 +779,11 @@ static int write_init_super1(struct supertype *st, void *sbv,
 		if (memcmp(sb->set_uuid, refsb->set_uuid, 16)==0) {
 			/* same array, so preserve events and dev_number */
 			sb->events = refsb->events;
-			sb->dev_number = refsb->dev_number;
+			/* bugs in 2.6.17 and earlier mean the dev_number
+			 * chosen in Manage must be preserved
+			 */
+			if (get_linux_version() >= 2006018)
+				sb->dev_number = refsb->dev_number;
 		}
 		free(refsb);
 	}
